#! /usr/bin/python

import sys
import numpy as np
import scipy.io
import os.path
sys.path.append(
    os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir)))
import denoise

def objective(b, y, X, gamma, alpha):
   return 0.5*np.sum((y-np.dot(X, b))**2) + 0.5*alpha*np.sum(b**2) + gamma*np.sum(abs(b))

# first make the stopping condition right

tol = 1e-10

for dat in ["sdatum", "datum"]:
   dic = scipy.io.loadmat("./test/{}.mat".format(dat))
   m = dic["m"][0][0]
   n = dic["n"][0][0]
   gamma = dic["gamma"][0][0]
   alpha = dic["alpha"][0][0]
   y = dic["y"]
   X = dic["X"]
   b = dic["b"]

   test_cases = [False, True] if dat is "sdatum" else [False]
   for case in test_cases:
      print("\ntest shooting with {} with standard_inputs = {}".format(dat, case))
      w = denoise.denoise.shootingEnet(y, X, gamma, alpha, tol=tol, standard_inputs=case)
      myobj = objective(w, y, X, gamma, alpha)
      cvxobj = objective(b, y, X, gamma, alpha)
      print("\nshooting gives {} compared to {} by CVX".format(myobj, cvxobj))
      print("{} wins by a margin {:.2E}!".format("shooting" if myobj < cvxobj else "CVX", abs(myobj - cvxobj)))
      agreement = np.linalg.norm(b-w)/np.linalg.norm(b+w)
      print("the solution agreement is {:.2E}".format(agreement))
      if agreement > 1e-5:
         print('error: shooting disgrees with CVX.')
         sys.exit(1)
